mod api;
mod collectors;
mod lifecycle;
mod transport;

use axum::{Router, BoxError};
use axum::routing::get;
use axum_tracing_opentelemetry::{response_with_trace_layer, opentelemetry_tracing_layer};
use std::net::SocketAddr;
use api::endpoints::stats;
use api::endpoints::worlds;

#[tokio::main]
async fn main() -> Result<(), BoxError> {
    axum_tracing_opentelemetry::tracing_subscriber_ext::init_subscribers()?;

    let app = Router::new()
        .route("/worlds", get(worlds::get_worlds))
        .route("/stats/:rsn", get(stats::get_stats))
        .layer(response_with_trace_layer())
        .layer(opentelemetry_tracing_layer());

    let addr = SocketAddr::from(([0, 0, 0, 0], 3030));

    tracing::warn!("listening on {}", addr);
    axum::Server::bind(&"0.0.0.0:3030".parse().unwrap())
        .serve(app.into_make_service())
        .with_graceful_shutdown(lifecycle::shutdown::shutdown_signal())
        .await?;

    Ok(())
}
