use reqwest::{Request, Response};
use reqwest_middleware::{ClientBuilder, ClientWithMiddleware};
use reqwest_middleware::{Middleware, Next, Result};
use reqwest_retry::{RetryTransientMiddleware, policies::ExponentialBackoff};
use reqwest_tracing::TracingMiddleware;
use async_trait::async_trait;

pub const USER_AGENT: &str = "osrs-prometheus-exporter";

pub fn new() -> ClientWithMiddleware {
    let retry_policy = ExponentialBackoff::builder().build_with_max_retries(3);

    ClientBuilder::new(reqwest::Client::new())
        // Add default headers
        .with(DefaultHeaderMiddleware::new())
        // Trace HTTP requests. See the tracing crate to make use of these traces.
        .with(TracingMiddleware::default())
        // Retry failed requests.
        .with(RetryTransientMiddleware::new_with_policy(retry_policy))
        .build()
}

struct DefaultHeaderMiddleware {}

impl DefaultHeaderMiddleware {
    fn new() -> Self {
        DefaultHeaderMiddleware{}
    }
}

#[async_trait]
impl Middleware for DefaultHeaderMiddleware {
    async fn handle(&self, mut req: Request, extensions: &mut task_local_extensions::Extensions, next: Next<'_>) -> Result<Response> {
        req.headers_mut().insert("User-Agent", USER_AGENT.parse().unwrap());
        next.run(req, extensions).await
    }
}
